/*!
 * Copyright (c) 2019 digitalSTROM AG, Switzerland
 *
 * \author Danilo Zanatta, danilo.zanatta@digitalstrom.com
 */

#ifndef DSF_H_
#define DSF_H_

#include <inttypes.h>
#include <string>
#include <list>
#include <vector>

#include <digitalSTROM/dsm-api-v2/dsm-api.h>

/*!
 * \class DSF
 * \brief dSF abstraction; takes care of all the ds485 communication
 */
class DSF
{
public:
    typedef enum {
        PHASE_ALL = 0,
        PHASE_1 = 1,
        PHASE_2 = 2,
        PHASE_3 = 3,
    } Phase;

    enum dSFReturnValues {
        DSF_ERROR_NO_CONFIG = 0x15,
    };

    typedef struct {
        dsuid_t     dsuid;
        std::string description;
    } Device;

    DSF();
    ~DSF();

    bool open(const std::string& connection);
    void close();

    bool getDeviceInfo(dsuid_t dsuid, DSF::Device &device);
    bool listDevices(std::list<Device>& devices);
    bool getVersion(const dsuid_t& dsuid,
                    uint32_t* hwVersion,
                    uint32_t* armSwVersion,
                    uint32_t* dspSwVersion,
                    uint32_t* barmSwVersion,
                    uint16_t* apiVersion,
                    std::string& name);

    bool readDSPConfig(const Device &device, const Phase phase, std::vector<uint8_t> &vConfig);
    bool writeDSPConfig(const Device &device, const Phase phase, const std::vector<uint8_t> &vConfig);
    bool restartAllDSPs(const Device &device);

    static bool parseDSUIDFromString(dsuid_t& dsuid, const std::string& dsuidString);
    static void prettyPrintDSUID(const dsuid_t& dsuid, std::string& dsuidString);

private:
    void init();

    bool m_opened;
    DsmApiHandle_t m_dsmApiHandle;
};

#endif /* DSF_H_ */
